#!@@@perl@@@

=head1 NAME

disk_checker - check for nearly full disks

=head1 SYNOPSIS

disk_checker

=head1 DESCRIPTION

The disk_checker script checks used space on all locally mounted
filesystems and sends a warning mail to the administrator if either the
amount of used space or free space changes by more than a given
percentage.

=head2 Configuration

Configuration is read from /etc/disk_checker, if it exists.
The configuration file is in yaml format.
There are currently two configuration options:

=over

=item threshold

Changes by more then this percentage cause a report

=item mailto

Reports are sent to these email-addresses.

=back

Configuration opions can be both global or per file system.

Example:

 ---
 /home:
   threshold: 50
 /static:
   threshold: 10

This sets the threshold to 50% for the /home filesystem and to 10% for
the /static file systems. All other file systems still have the default
threshold of 20%.


=head1 DESIGN NOTES

This script replaces an older script of the same name. Its
behaviour is substantially different, though:

=over

=item *

The script reports if the usage changes too much, not if it exceeds a
certain limit. When filesystem usage is constant (even if it is
something like 98 %) there is no need to be constantly reminded. But if
it changes drastically (in either direction) one should investigate.

=item *

There is no exponential backoff any more. Mails are only sent if there
is a significant change, so it shouldn't be necessary (didn't work very
well anyway).

=item *

All the configuration is now done in a singe config file.

=item *

The state cache now resides in /var/tmp instead of /var/run, so no
special privileges are necessary to set it up. There is however, a
slight probability of interference from other users.

=back

=head1 AUTHOR

Peter J. Holzer, <hjp@wsr.ac.at>

=head1 SEE ALSO

df(1)

=cut

use v5.10;
use strict;
use warnings;
use POSIX;
use Storable;
use YAML qw(LoadFile);

# config section (sort of)
my $statedir = "/var/tmp/disk_checker";
mkdir $statedir;
my $statefile = "$statedir/state";

open my $log_fh, '>>', "$statedir/" . strftime("%Y-%m-%d.log", localtime);


# the works
my %maxpct;

my $cfg;
if (-f "/etc/disk_checker") {
    $cfg = LoadFile("/etc/disk_checker");
}

my $mailto='system@wsr.ac.at';
if ($cfg->{mailto}) {
    $mailto = $cfg->{mailto};
}

my $global_threshold = 20;
if ($cfg->{threshold}) {
    $global_threshold = $cfg->{threshold};
}

my $state = eval { retrieve($statefile) };
my $msg = "";

open (DF, '@@@df-Plm@@@ |') or die 'cannot invoke @@@df-Plm@@@: '.$!;
while (<DF>) {
    my ($fs, $blk, $used, $avail, $pct, $mnt) = split;
    next unless ($fs =~ m|/dev/|);
    $pct =~ s/%//;
    my $threshold = ($cfg->{$mnt}{threshold} // $global_threshold) / 100;
    say $log_fh strftime("%Y-%m-%d %H:%M:%S", localtime), " $mnt $used $avail $pct $threshold";
    if (abs($used - $state->{$mnt}{used}) > $state->{$mnt}{used} * $threshold
        || abs($avail - $state->{$mnt}{avail}) > $state->{$mnt}{avail} * $threshold
        || abs($pct - $state->{$mnt}{pct}) > $state->{$mnt}{pct} * $threshold
    ) {

	$msg .= sprintf("-%s %7d %7d %3d%% %s\n",
                        strftime("%Y-%m-%d %H:%M", localtime($state->{$mnt}{ts})),
                        $state->{$mnt}{used},
                        $state->{$mnt}{avail},
                        $state->{$mnt}{pct},
                        $mnt,
                       );
	$msg .= sprintf("+%s %7d %7d %3d%% %s\n",
                        strftime("%Y-%m-%d %H:%M", localtime()),
                        $used,
                        $avail,
                        $pct,
                        $mnt,
                       );
        $state->{$mnt} = { used => $used, avail => $avail, pct => $pct, ts => time };
	if ($cfg->{$mnt}{mailto}) {
            $mailto .= ", " . $cfg->{$mnt}{mailto};
	}
    }
}
close (DF) or die 'cannot close @@@df-Plm@@@: '.$!;

if ($msg eq "") {
    exit(0);
}

my ($sysname, $nodename, $release, $version, $machine ) =
    POSIX::uname();

open (SENDMAIL, '|@@@sendmail@@@ -i -t') or die 'cannot invoke @@@sendmail@@@: '.$!;
print SENDMAIL "To: $mailto\n";
print SENDMAIL "Subject: Filesystems over limit on $nodename\n";
print SENDMAIL "\n";
print SENDMAIL "$msg";

close (SENDMAIL) or die 'cannot close @@@sendmail@@@: '.$!;

store $state, $statefile;

# vim:sw=4 tw=132
